#!/usr/local/bin/ruby

class Integer
	def cshl(x, b = 32)
		return ((self << x) | (self >> (b - x))) & ((1 << b) - 1)
	end
end

class SHA1
	def initialize(message = '')
		@message = message
		padmessage
		@aA, @bB, @cC, @dD, @eE = A, B, C, D, E
		return self
	end
	def message=(string)
		@message = string.to_s
		padmessage
	end
	def hash
		blocks = @message.length / 64 - 1
		for block in 0..blocks
			block *= 64
			hashblock @message[block..(block + 63)]
		end
		return self
	end
	def finish
		ret = ''
		[@aA, @bB, @cC, @dD, @eE].each {|v| ret += format("%08x", v) }
		return ret
	end
	private
	def hashblock(m)
		ops = [
		   [lambda {|x, y, z| f0(x, y, z) }, K0],
		   [lambda {|x, y, z| f1(x, y, z) }, K1],
		   [lambda {|x, y, z| f2(x, y, z) }, K2],
		   [lambda {|x, y, z| f3(x, y, z) }, K3]
		]
		w = transblock m
		a, b, c, d, e = @aA, @bB, @cC, @dD, @eE
		for t in 0..79
			op = ops[t / 20]
			temp = (a.cshl(5) + op[0].call(b, c, d) + e +
			   w[t] + op[1]) & 0xffffffff
			e = d
			d = c
			c = b.cshl(30)
			b = a
			a = temp
		end
		@aA = (@aA + a) & 0xffffffff
		@bB = (@bB + b) & 0xffffffff
		@cC = (@cC + c) & 0xffffffff
		@dD = (@dD + d) & 0xffffffff
		@eE = (@eE + e) & 0xffffffff
	end
	def transblock(m)
		w = Array.new
		for t in 0..15
			offset = t * 4
			w[t] = m[offset..(offset + 3)].unpack('I')[0]
		end
		for t in 16..79
			wtemp = w[t - 3] ^ w[t - 8] ^ w[t - 14] ^ w[t - 16]
			w[t] = wtemp.cshl(1)
		end
		w.freeze
		return w
	end
	def padmessage
		oldlen = @message.length * 8
		@message += 0x80.to_a.pack('C')
		padlen = 64 - ((@message.length + 8) % 64)
		if padlen != 64
			@message += [0].pack('C') * padlen
		end
		@message += [(oldlen & 0xffffffff00000000) >> 32,
		   oldlen & 0xffffffff].pack('NN')
		words = @message.length / 4 - 1
		for word in 0..words
			word *= 4
			@message[word..(word + 3)] =
			   @message[word..(word + 3)].unpack('N').pack('I')
		end
		@message.freeze
	end
	A = 0x67452301
	B = 0xefcdab89
	C = 0x98badcfe
	D = 0x10325476
	E = 0xc3d2e1f0
	def f0(x, y, z)
		return (x & y) | ((~x) & z)
	end
	K0 = 0x5a827999
	def f1(x, y, z)
		return x ^ y ^ z
	end
	K1 = 0x6ed9eba1
	def f2(x, y, z)
		return (x & y) | (x & z) | (y & z)
	end
	K2 = 0x8f1bbcdc
	alias f3 f1
	K3 = 0xca62c1d6
end

print SHA1.new(STDIN.read || '').hash.finish, "\n"
